/**
 * 	#####################################################################################	
 * 
 * 	Custom logger module
 * 
 * 	#####################################################################################
 */
var util   = require('util');
var mailer = require('mdlib-mailer');

/** -------------------------------------------------------------------------------------
 * 	Definition log levels
 * 	-------------------------------------------------------------------------------------
 */
var logLevels = { 
	fatal: 		{"level": 0, "color": "95", "name": "fatal" }, 
	error: 		{"level": 1, "color": "91", "name": "error"}, 
	warn: 		{"level": 2, "color": "93", "name": "warn"},
	ok: 		{"level": 3, "color": "92", "name": "ok"}, 
	info: 		{"level": 4, "color": "97", "name": "info"},
	debug: 		{"level": 5, "color": "96", "name": "debug"},
};

/**
 * 	function: 		formatLogDate
 * 	description: 	- formats a date object in a standard output
 */	
var _formatLogDate = function(date, withoutSeconds){
	var day 	= (date.getDate() < 10) ? "0" + date.getDate() : date.getDate();
	var month 	= (date.getMonth() < 10) ? "0" + (date.getMonth() + 1) : (date.getMonth() + 1);
	var hour 	= (date.getHours() < 10) ? "0" + date.getHours() : date.getHours();
	var minute	= (date.getMinutes() < 10) ? "0" + date.getMinutes() : date.getMinutes();
	var second	= (date.getSeconds() < 10) ? "0" + date.getSeconds() : date.getSeconds();
	var s = day + "." + month + "." + date.getFullYear() + " " + hour + ":" + minute;
	s += (withoutSeconds) ? "" : ":" + second;
	return s;
};	

/**
 * function: 		sendMail
 * description: 	- send an email with log informations
 */	
var _sendLogMail = function(options, value, metadata, logLevel){	
	var moduleStr = (options.module) ? options.module + ' ::: ' : '';
	if(typeof value === "object"){
		var subject = "["+ logLevel.name.toUpperCase() +" LOG] - " + value.message + " (Code: " + value.code + ")"; 		
		var mailmsg = _formatLogDate(new Date()) + ' ::: ' + logLevel.name.toUpperCase() + ' ::: ' + moduleStr + value.message + ' (Code: ' + value.code + ') ';
	}else{
		var subject = "["+ logLevel.name.toUpperCase() +" LOG] - " + value; 		
		var mailmsg = _formatLogDate(new Date()) + ' ::: ' + logLevel.name.toUpperCase() + ' ::: ' + moduleStr + value;
	}

	if(metadata){ mailmsg += '<br>' + util.inspect(metadata); }	
	switch(options.type){
		case "mandrill":
			mailer.mandrill(options).send(options.from, options.to, subject, options.name, mailmsg);
			break;
		case "sendgrid": 	
			//utilsMail.sendgrid.send(options.from, options.to, subject, options.name, msg); 
			break;	
	}
};

var _log = function(options, value, metadata, withmail, logLevel){
	var moduleStr = (options.module) ? options.module + ' ::: ' : '';
	if(typeof value === "object"){
		var logStr = _formatLogDate(new Date()) + ' ::: ' + logLevel.name.toUpperCase() + ' ::: ' + moduleStr + value.message + ' (Code: ' + value.code + ') ';	
	}else{
		var logStr = _formatLogDate(new Date()) + ' ::: ' + logLevel.name.toUpperCase() + ' ::: ' + moduleStr + value;
	}				
	if(metadata){ logStr += '\n' + util.inspect(metadata); }		
	
	if(logLevel.level < 3){
		console.error('\033['+logLevel.color+'m' + logStr + '\033[m'); 
	}else{
		console.log('\033['+logLevel.color+'m' + logStr + '\033[m');
	}	
	if(options && options.mail){
		if(typeof withmail === 'boolean' && !withmail){ return; }  
		if(typeof withmail === 'boolean' && withmail){ return _sendLogMail(options, value, metadata, logLevel); }  
		if(withmail){ return _sendLogMail(options.mail, value, metadata, logLevel); }  		
		if(options.mail.levels === 'all'){ return _sendLogMail(options.mail, value, metadata, logLevel); }	
		if(Array.isArray(options.mail.levels) && options.mail.levels.indexOf(logLevel.name) != -1){ return _sendLogMail(options.mail, value, metadata, logLevel); }	
		if(logLevel.level < 3){ return _sendLogMail(options.mail, value, metadata, logLevel); }	
	}
};

var CLogger = {
	
	/*
	 * 	options = {
	 * 		mail: {
	 * 			levels: [],				// default: ['fatal', 'error', 'warn']; value: "all" || ['fatal', 'error', 'warn', 'ok', 'info', 'debug']
	 * 			type: 'mandrill',
	 * 			key: '<key>', 
	 * 			from: '<email adress>', 
	 * 			name: '<name>',
	 * 			to: '<email adress>',	
	 * 		}
	 * 	}
	 */
	
  	getLogger: function (options, module) {
  		options.module = module;
   	 	return {
	  		fatal: 	function (value, metadata, withmail) { _log(options, value, metadata, withmail, logLevels.fatal); },
	  		error: 	function (value, metadata, withmail) { _log(options, value, metadata, withmail, logLevels.error); },
	  		warn: 	function (value, metadata, withmail) { _log(options, value, metadata, withmail, logLevels.warn); },
	  		ok: 	function (value, metadata, withmail) { _log(options, value, metadata, withmail, logLevels.ok); },
	  		info: 	function (value, metadata, withmail) { _log(options, value, metadata, withmail, logLevels.info); },
	  		debug: 	function (value, metadata, withmail) { _log(options, value, metadata, withmail, logLevels.debug); },
    	};
  	}	
};

module.exports = CLogger;